//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : main.c
//* Object              : main application written in C
//* Creation            : FB   21/11/2002
//*
//*----------------------------------------------------------------------------

/* modified in 2004-2005 by Dmitriy Cherkashin */
/* dch@ucrouter.ru,  www.ucrouter.ru */
/* based on ATMEL AT91RM9200-BasicMCIDevice code */

#include "mci_device.h"

#define EVM9200								/* comment to AT91RM9200-DK */
//#define MCI_CID_INFO
//#define MCI_OCR_INFO
#define MCI_R1_INFO
//#define MCI_CSD_INFO
//#define MCI_CSD_DEBUG
//#define MCI_DEBUG

#define AT91C_MCI_TIMEOUT		1000000   /* For AT91F_MCIDeviceWaitReady */
#define BUFFER_SIZE_MCI_DEVICE		512

//* External Functions
extern void			AT91F_DBGU_Printk(char *);
extern void			AT91F_DBGU_Printl(unsigned long val);
extern void			AT91F_DBGU_Printx(unsigned long val);
extern void			AT91F_DBGU_Printv(char * pref, unsigned long val, char *suf);
extern void			AT91F_DBGU_Printw(char * pref, unsigned long val, char *suf);
extern int			AT91F_DBGU_getch(char * ch);
extern void			AT91F_ASM_MCI_Handler(void);
extern void			AT91F_MCI_Device_Handler(AT91PS_MciDevice,unsigned int);
#ifdef SDCARD_TEST 
extern AT91S_MCIDeviceStatus 	AT91F_MCI_SDCard_Init (AT91PS_MciDevice);
#endif 															/* SDCARD_TEST */
extern AT91S_MCIDeviceStatus	AT91F_MCI_MMC_Init(AT91PS_MciDevice);
extern AT91S_MCIDeviceStatus 	AT91F_MCI_SetBlocklength(AT91PS_MciDevice,unsigned int);
extern AT91S_MCIDeviceStatus 	AT91F_MCI_MMC_SelectCard(AT91PS_MciDevice,unsigned int);
extern AT91S_MCIDeviceStatus 	AT91F_MCI_ReadBlock(AT91PS_MciDevice,int,unsigned int *,int);
extern AT91S_MCIDeviceStatus 	AT91F_MCI_WriteBlock(AT91PS_MciDevice,int,unsigned int *,int);
extern AT91S_MCIDeviceStatus    AT91F_MCI_SendCommand (
		AT91PS_MciDevice pMCI_Device, unsigned int Cmd, 
		unsigned int Arg, unsigned int * response );

//* Global Variables
AT91S_MciDevice	MCI_Device;
char            Buffer[BUFFER_SIZE_MCI_DEVICE];

unsigned long master_clock = 0;			/* MASTER clock */
unsigned long sdram_size = 0;			/* SDRAM size */

//*----------------------------------------------------------------------------
//* \fn    AT91F_MCIDeviceWaitReady
//* \brief Wait for MCI Device ready
//*----------------------------------------------------------------------------
int AT91F_MCIDeviceWaitReady(unsigned int timeout)
{
 volatile int status;
	
 do {
    status = AT91C_BASE_MCI->MCI_SR;
    timeout--;
 } while( !(status & AT91C_MCI_NOTBUSY)  && (timeout>0) );	
	
 status = AT91C_BASE_MCI->MCI_SR;
 if( (status & AT91C_MCI_NOTBUSY) == 0 ) {
#ifdef MCI_DEBUG 
    AT91F_DBGU_Printk("MMC"
#ifdef SDCARD_TEST
     "/SD"
#endif 							/* SDCARD_TEST */
     " Card not ready for transfer \n\r");
#endif 															/* MCI_DEBUG */
    return(0);						/* not ready for data transfer */
 } else {
#ifdef MCI_DEBUG 
    AT91F_DBGU_Printk("MMC"
#ifdef SDCARD_TEST
     "/SD"
#endif 							/* SDCARD_TEST */
     " Card ready for transfer \n\r");
#endif 															/* MCI_DEBUG */
    return(1);						/* ready for data transfer */
 } 
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_Test
//* \brief Test Functions
//*----------------------------------------------------------------------------
int AT91F_Test(void)
{
 static int test_num = 0;
 unsigned int len;
 int i;

 /* test number */
 test_num++;

 //* ReadBlock & WriteBlock Test -> Entire Block

 /* not correctly read block len may be not equal write block length and */
 /* it possible what device not support misalignment read/write */
 len = MCI_Device.card[0].Max_Read_DataBlock_Length;
 if(len > MCI_Device.card[0].Max_Write_DataBlock_Length) {
    len = MCI_Device.card[0].Max_Write_DataBlock_Length;
 } 

 //* Wait MCI Device Ready
 AT91F_MCIDeviceWaitReady(AT91C_MCI_TIMEOUT);

 //* Read Block 1
 for(i=0;i<BUFFER_SIZE_MCI_DEVICE;i++) {
    Buffer[i] = 0x00;	
 }

 AT91F_MCI_ReadBlock(&MCI_Device,(1*len),(unsigned int*) Buffer,len);

 //* Wait end of Read
 AT91F_MCIDeviceWaitReady(AT91C_MCI_TIMEOUT);

 //* Write block 1
 {
    char * src = "This sentence is written in your device... Congratulations\n\r";
    int dpos = 0;
    int spos = 0;

    Buffer[dpos++] = '\n';
    Buffer[dpos++] = '\r';
    Buffer[dpos++] = '0' + (test_num / 100) % 10;	
    Buffer[dpos++] = '0' + (test_num / 10) % 10;		
    Buffer[dpos++] = '0' + (test_num % 10);			
    Buffer[dpos++] = ' ';

    while(1) {
	Buffer[dpos] = *(src + spos);
	if(*(src + spos) == '\0') {
	    break;
	}
	dpos++;
	spos++;
	if(dpos + 1 >= sizeof(Buffer)) {
 	    Buffer[dpos] = '\n';
	    break;
	}
    }
 }

 AT91F_MCI_WriteBlock(&MCI_Device,(0*len),(unsigned int*) Buffer, len);

 //* Wait end of Write
 AT91F_MCIDeviceWaitReady(AT91C_MCI_TIMEOUT);

 //* Read Block 1
 for(i=0;i<BUFFER_SIZE_MCI_DEVICE;i++) {
    Buffer[i] = 0x00;
 }

 AT91F_MCI_ReadBlock(&MCI_Device,(0*len),(unsigned int*) Buffer, len);

 //* Wait end of Read
 AT91F_MCIDeviceWaitReady(AT91C_MCI_TIMEOUT);

 //* End Of Test
 AT91F_DBGU_Printk("\n\rTests Completed: !!!\n\r");
 AT91F_DBGU_Printk(Buffer);

 return 1;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_CfgDevice
//* \brief This function is used to initialise MMC or SDCard Features
//*----------------------------------------------------------------------------
void AT91F_CfgDevice(void)
{
 int dn;

 // Init Device Structure
 MCI_Device.state	     = AT91C_MCI_IDLE;
 MCI_Device.SDCard_bus_width = AT91C_MCI_SCDBUS;
 MCI_Device.card_num	     = 0;

 for(dn = 0; dn < AT91C_MAX_MCI_CARDS; ++dn) {
    MCI_Device.card[dn].Relative_Card_Address      = 0;
    MCI_Device.card[dn].Card_Inserted              = AT91C_CARD_REMOVED;
    MCI_Device.card[dn].Max_Read_DataBlock_Length  = 0;
    MCI_Device.card[dn].Max_Write_DataBlock_Length = 0;
    MCI_Device.card[dn].Read_Partial               = 0;
    MCI_Device.card[dn].Write_Partial              = 0;
    MCI_Device.card[dn].Erase_Block_Enable         = 0;
    MCI_Device.card[dn].Sector_Size                = 0;
    MCI_Device.card[dn].Memory_Capacity            = 0;
 }
}

/* print command response */

int AT91F_MCI_Resp_Info(unsigned int * resp)
{
 int i;
 if(resp == 0) {
	return(0);
 }

 for(i = 0; i < 4; ++i) {
    unsigned char ch;
    char buf[20];
    int in;
    int bn;

    /* 8 characters */
    bn = 0;
    for(in = 28; in >= 0; in -= 4) {
	ch = ((*(resp + i) >> in) & 0xF);
	if(ch <= 9) {
	    ch += '0'; 
	} else {
	    ch = 'A' + (ch - 10);
	}
	buf[bn++] = ch;
    }

    /* + 1 space */			
    buf[bn++] = ' ';

    /* +4 char */
    for(in = 24; in >= 0; in -= 8) {
	ch = (( *(resp + i) >> in) & 0xFF);
	if(ch >= 32 && ch < 128) {
	    buf[bn++] = ch;
	} else {
	    buf[bn++] = '.';
	}
    }

    /* +3 end char */
    buf[bn++] = '\n';
    buf[bn++] = '\r';
    buf[bn++] = '\0';

    /* 8 + 1 + 4 + 3 = 16 */
    AT91F_DBGU_Printk(buf);
 }

 return(1);
}
/* print OCR (operation condition register) content */

int AT91F_MCI_OCR_Info(unsigned int * response)
{
#ifdef MCI_OCR_INFO
 int v0, v1;						/* voltage level */
 int bn;							/* bit number */
#endif 														/* MCI_OCR_INFO */

 if(response == 0) {
    return(0);
 }	


#ifdef MCI_OCR_INFO
 v0 = 145;
 for(bn = 0; bn < 24; ++bn) {
    if(bn < 5) {
	v1 = v0 + 5;
    } else {
	v1 = v0 + 10;
    }

    AT91F_DBGU_Printl(v0);
    AT91F_DBGU_Printk("-");
    AT91F_DBGU_Printl(v1);
    if( (*response & (1 << bn)) != 0) {
	AT91F_DBGU_Printk("+");
    } else {
	AT91F_DBGU_Printk(" ");
    }

    if( ((bn + 1) % 8) == 0 ) {
	AT91F_DBGU_Printk("\n\r");
    }
    v0 = v1;
 }	

 if( (*response & AT91C_CARD_POWER_UP_BUSY) == 0 ) {
    AT91F_DBGU_Printk("Card has not finished the power up routine\n\r");
 }
//
#endif 														/* MCI_OCR_INFO */
 return(1);
}

/* print CID Card Indentification Register and Exit  */

int AT91F_MCI_CID_Info(unsigned int * response)
{
 if(response == 0) {
    return(0);
 }

#ifdef MCI_CID_INFO
/* Manufacturer ID MID 8 [127:120] */
 AT91F_DBGU_Printw("MID (Manufacturer ID)     8 bit", ((*response >> 24) & 0xFF),"");
/* OEM/Application ID OID 16 [119:104] */
 AT91F_DBGU_Printw("OID (OEM/Application ID) 16 bit", ((*response >> 8) & 0xFFFF),"");
/* Product name PNM 48 [103:56] */
 {
    char buf[7];

 AT91F_DBGU_Printk("PNM (Product name) 48 bits     ");
    buf[0] =  (*response & 0xFF);
    buf[1] = (*(response+1) >> 24) & 0xFF;
    buf[2] = (*(response+1) >> 16) & 0xFF;
    buf[3] = (*(response+1) >>  8) & 0xFF;
    buf[4] = (*(response+1) >>  0) & 0xFF;
    buf[5] = (*(response+2) >> 24) & 0xFF;
    buf[6] = '\0';

    AT91F_DBGU_Printk(buf);
    AT91F_DBGU_Printk("\n\r");
 }

/* [55:48] */
 AT91F_DBGU_Printw("PRV Product revision 8 bits    ", ((*(response +2) >> 16) & 0xFF),"");
 AT91F_DBGU_Printw("PSN Pr serial number 32bits    ", 
    ( ((*(response +2) & 0xFFFF) << 16) | ((*(response +3) >> 16) & 0xFFFF) ),
    "\n\r");

/* MDT Manufacturing date 8 bits [15:8] */
 AT91F_DBGU_Printk("MDT Manufacturing date 8 bits  ");

 AT91F_DBGU_Printl( ((*(response +3) >> 12) & 0xF) );
 AT91F_DBGU_Printk(".");
 AT91F_DBGU_Printl( ((*(response +3) >>  8) & 0xF) + 1997 );
 AT91F_DBGU_Printk("\n\r");

 AT91F_DBGU_Printw("CRC7 checksum CRC 7 bits       ", (*(response +3) >> 1) & 0x7F, "" );
 AT91F_DBGU_Printv("not used, always 1 (1 bit)     ", (*(response +3) & 0x1), "\n\r" );
#endif														/* MCI_CID_INFO */

 return(1);
}

/* print not zero bit */

int AT91F_DBGU_Printb(char * mes, unsigned int val, int bn, char * suf) 
{
 int pos = 0;						/* current screen position */

 if(mes != 0) { 
    char buf[3];

    buf[2] = '\0';

    if(bn > 9) {
	buf[0] = '0' + (bn / 10);
	buf[1] = '0' + (bn % 10);
    } else {
	buf[0] = ' ';
	buf[1] = '0' + bn;
    } 

    AT91F_DBGU_Printk( buf );		
    AT91F_DBGU_Printk( " " );
    AT91F_DBGU_Printk( mes );

    pos = 0;
    while( *(mes + pos) != '\0' ) {
	pos++;
    }

/* */
    pos += 3;
 } 

/* */
 while(pos < 40) {
    AT91F_DBGU_Printk( "." );
    pos++;
 }

 val = (val >> bn) & 0x1;
 if( val ) {
    AT91F_DBGU_Printk( "1" );
 } else {
    AT91F_DBGU_Printk( "0" );
 }
//
 if(suf == 0) {
    AT91F_DBGU_Printk("\n\r");
 } else {
    AT91F_DBGU_Printk(suf);
 }
 return(val);
}

int AT91F_DBGU_Printbc(char * mes, unsigned int val, int bn, char * suf)
{
 if( (val & (1 << bn)) != 0 ) { 
    return( AT91F_DBGU_Printb(mes, val, bn, suf) ); 
 } else {
    return(1);
 }
}

/* print bit field */

int AT91F_DBGU_Printf(char * mes, unsigned int val, int off, int msk, char * suf) 
{
 int pos = 0;

 {
    int tmsk = msk;
    int flen = 0;
    while( (tmsk & 1) != 0 ) {
    	tmsk >>= 1;
	flen += 1;
    } 
    if(flen == 0) {
	flen = 1;
    }

    flen = off + flen - 1;
    AT91F_DBGU_Printl(flen);
    AT91F_DBGU_Printk(":");
    if(off == 0) {
	AT91F_DBGU_Printk("0");
    } else {
	AT91F_DBGU_Printl(off);
    }
    AT91F_DBGU_Printk(" ");

    pos += 2;
    if(flen > 99) { pos += 3; } else
    if(flen >  9) { pos += 2; } else
     { pos += 1; }

    if(off >  99) { pos += 3; } else
    if(off >   9) { pos += 2; } else
     { pos += 1; }
 }

 if(mes != 0) { 
    int mpos = 0;
    AT91F_DBGU_Printk( mes );
    while( *(mes + mpos) != '\0' ) {
	mpos++;
	pos++;
    }
 } 

 while(pos < 40) {
    AT91F_DBGU_Printk( "." );
    pos++;
 }

 val = ((val >> off) & msk);
 AT91F_DBGU_Printk("0x");
 {
    unsigned int tmp;
    unsigned char ch;
    char buf[9]; 

    tmp = val;
    pos = 7;
    buf[8] = '\0';
    while(1) {
	ch = (tmp & 0xF); 
	tmp = (tmp >> 4);
	if(/* ch >= 0 && */ ch <= 9) {
	    buf[pos] = '0' + ch;
	} else {
    	    buf[pos] = 'A' + (ch - 10);
	}

	if(tmp == 0) {
    	    break;
	} else {
	    pos--;
	}
    }

    AT91F_DBGU_Printk(buf + pos);
 }

//
 if(suf != 0) {
    AT91F_DBGU_Printk(suf);
 } else {
    AT91F_DBGU_Printk("\r\n");
 }	

 return( val );
}

/* dump buffer */
int AT91F_DBGU_Dump(char * buffer, unsigned int off, unsigned int bytes) 
{
 int bpos;				/* buffer position */
 int row;				/* row number */
 
 if(buffer == 0) {				
    return(0);
 }

 bpos = 0; 				/* start buffer position */
 row = 0;				/* start row number */
 while( bpos < bytes ) {
    char buf[2];			/* temp buffer */	
    char ch;				/* printed character */
    int pos;
    int cnt;

    buf[1] = 0;				/* string terminator */

    /* print offset */
    for(pos = 28; pos >= 0; pos -= 4) {
	ch = (off >> pos) & 0xF;
	if(ch >= 0 && ch <= 9) {
	    ch += '0';
	} else {
	    ch  = 'A' + (ch - 10); 
	} 
	buf[0] = ch;
 	AT91F_DBGU_Printk( buf );
    }	
	
    /* offset and hex dump separator */
    AT91F_DBGU_Printk( " " );

    /* print dump line */
    cnt = 16;
    if(cnt > bytes - bpos) {
	cnt = bytes - bpos;
    }

    for(pos = 0; pos < cnt; ++pos) {
	char byte;

	/* read byte */
	byte = *(buffer + bpos + pos);
	
	ch = ((byte >> 4) & 0xF);
	if(ch >= 0 && ch <= 9) {
	    ch += '0';
	} else {
    	    ch  = 'A' + (ch - 10);
	} 
	buf[0] = ch;	
 	AT91F_DBGU_Printk( buf );

	ch = (byte & 0xF);
	if(ch >= 0 && ch <= 9) {
	    ch += '0';
	} else {
	    ch  = 'A' + (ch - 10);
	} 
	buf[0] = ch;	
 	AT91F_DBGU_Printk( buf );
    }

    /* hex dump and ascii dump separator */ 
    AT91F_DBGU_Printk( " " );

    for(pos = 0; pos < cnt; ++pos) {
	char byte;

	/* read byte */
	byte = *(buffer + bpos + pos);
	if(byte >= 32 && byte < 128) {
	    buf[0] = byte;
	} else {
	    buf[0] = '.';
	}
 	AT91F_DBGU_Printk( buf );
    }

    bpos += cnt;			/* increment buffer position */
    off  += cnt;			/* increment offset */ 

/* */
    AT91F_DBGU_Printk( "\n\r" );

    row++; 
    if(row >= 16) {
	row = 0; 
	while(1) {
    	    char ch;
	    if(AT91F_DBGU_getch(&ch)) {
		break;
	    }
 	}
    }
 }
 return(1);
}

/* print R1 response content */
int AT91F_MCI_R1_Info(unsigned int * resp)
{
 unsigned int val;
 if(resp == 0) {
    return(0);
 }

#ifdef MCI_R1_INFO
/* the command arg was out of allowed range for the card */
 val = *resp;
 AT91F_DBGU_Printbc("OUT_OF_RANGE"     , val,31,0);
 AT91F_DBGU_Printbc("ADDRESS_ERROR"    , val,30,0); /* misaligned address */
 AT91F_DBGU_Printbc("BLOCK_LEN_ERROR"  , val,29,0); /* transferred block length error */
 AT91F_DBGU_Printbc("ERASE_SEQ_ERROR"  , val,28,0); /* error in the sequence of erase */
 AT91F_DBGU_Printbc("ERASE_PARAM"      , val,27,0); /* an invalid selection of erase groups for erase command */
 AT91F_DBGU_Printbc("WP_VIOLATION"     , val,26,0); /* attempt to program a write protected block */
 AT91F_DBGU_Printbc("CARD_IS_LOCKED"   , val,25,0); /* card is locked */
 AT91F_DBGU_Printbc("LOCK_UNLOCK_FAILED",val,24,0); /* */
 AT91F_DBGU_Printbc("COM_CRC_ERROR"    , val,23,0); /* The CRC check of the previous command failed */
 AT91F_DBGU_Printbc("ILLEGAL_COMMAND"  , val,22,0); /* command not legal for the card state */
 AT91F_DBGU_Printbc("CARD_ECC_FAILED"  , val,21,0); /* card internal ECC was applied but failed to correct the data */
 AT91F_DBGU_Printbc("CC_ERROR"         , val,20,0); /* card controller error */
 AT91F_DBGU_Printbc("ERROR"            , val,19,0); /* general or an unknown error */
 AT91F_DBGU_Printbc("UNDERRUN"         , val,18,0); /* */
 AT91F_DBGU_Printbc("OVERRUN"          , val,17,0); /* */
 AT91F_DBGU_Printbc("CID_CSD_OVERWRITE", val,16,0);
 AT91F_DBGU_Printbc("WP_ERASE_SKIP"    , val,15,0);
 AT91F_DBGU_Printbc("CARD_ECC_DISABLED", val,14,0);
 AT91F_DBGU_Printbc("SR_ERASE_RESET"   , val,13,0);

 {
    char * mstr;
    switch( (val & AT91C_SR_STATE_MASK) ) {
	case AT91C_SR_IDLE  : mstr = "IDLE"; break; /* idle state */
	case AT91C_SR_READY : mstr = "READY";break; /* ready state */
	case AT91C_SR_IDENT : mstr = "IDENT";break; /* identification state */
	case AT91C_SR_STBY  : mstr = "STBY"; break; /* standby state */
	case AT91C_SR_TRAN  : mstr = "TRAN"; break; /* transfer state */
	case AT91C_SR_DATA  : mstr = "DATA"; break; /* sending date state */
	case AT91C_SR_RCV   : mstr = "RCV";  break; /* receiving data state */
	case AT91C_SR_PRG   : mstr = "PRG";  break; /* programming state */
	case AT91C_SR_DIS   : mstr = "DIS";  break; /* disconnect state */ 
	default :  mstr = "reserved"; break;
    }

    AT91F_DBGU_Printk("State ");
    AT91F_DBGU_Printk( mstr );
    AT91F_DBGU_Printk("\n\r");
 }

 AT91F_DBGU_Printbc("READY_FOR_DATA"   , val,8,0);
 AT91F_DBGU_Printbc("APP_CMD"          , val,5,0);
#endif 													/* MCI_R1_INFO */

//
 return(1);
}

/* print Card Specific Data Register content */

int AT91F_MCI_CSD_Info(unsigned int * resp)
{
#ifdef MCI_CSD_INFO
 unsigned int val;		/* response register value */
 unsigned int bvl;		/* bit field value  */
#endif 													/* MCI_CSD_INFO */

 if(resp == 0) {
    return(0);
 }

#ifdef MCI_CSD_INFO

/* RESP0 */
 val = *(resp + 0);

 {
/* CSD structure */	
    AT91F_DBGU_Printf( "CSD_STRUCT", val, AT91C_CSD_STRUCT_S, 
    	AT91C_CSD_STRUCT_M, 0); 
 }

 {
/* System Specification Version */
    char * vstr;

    bvl = AT91F_DBGU_Printf("SPEC_VERS" , val, AT91C_CSD_MMC_SPEC_VERS_S, 
		AT91C_CSD_MMC_SPEC_VERS_M, " ");

    switch(bvl) {
	case 0 : vstr = "1.0-1.2";  break;
	case 1 : vstr = "1.4"; 		break;
	case 2 : vstr = "2.0-2.2";  break;
	case 3 : vstr = "3.1";  	break;
	default: vstr = "reserved"; break;
    }
    AT91F_DBGU_Printk( vstr );
    AT91F_DBGU_Printk( "\n\r" );
 }

 {
/* read access time */
    static char * u_tacc[8] = {
	"1ns"	,	/* 0 */ 
	"10ns"	,	/* 1 */
	"100ns"	,	/* 2 */
	"1us"	,	/* 3 */
	"10us"	,	/* 4 */
	"100us"	,	/* 5 */
	"1ms"	,	/* 6 */
	"10ms"		/* 7 */
    };	

    static char *v_tacc[16] = {
    	"0"  ,		/* 0 */
	"1.0",		/* 1 */
	"1.2",		/* 2 */
	"1.3",		/* 3 */
	"1.5",		/* 4 */
	"2.0",		/* 5 */
	"2.5",		/* 6 */
	"3.0",		/* 7 */
	"3.5",		/* 8 */
	"4.0",		/* 9 */
	"4.5",		/* 10 */
	"5.0",		/* 11 */
	"5.5",		/* 12 */
	"6.0",		/* 13 */
	"7.0",		/* 14 */
	"8.0"		/* 15 */
    };


    bvl = AT91F_DBGU_Printf("TACC" , val, AT91C_CSD_TAAC_S, 
			AT91C_CSD_TAAC_M, " ");

    AT91F_DBGU_Printk( v_tacc[ (bvl >> 3) & 0xF ] );
    AT91F_DBGU_Printk( " " );
    AT91F_DBGU_Printk( u_tacc[ (bvl & 0x7) ] );
    AT91F_DBGU_Printk( "\n\r" );
 }

 {
    AT91F_DBGU_Printf( "NSAC", val, AT91C_CSD_NSAC_S, 
		AT91C_CSD_NSAC_M, 0);
 }

 {
    static char * u_speed[8] = { 
	"100kbit/s",	/* 0 */
	"1Mbit/s", 		/* 1 */
	"10Mbit/s", 	/* 2 */
	"100Mbit/s",	/* 3 */
	"","","",""
    };

    static char * v_speed[16] = { 
	"0"	,		/* 0 */
	"1.0",		/* 1 */
	"1.2",		/* 2 */
	"1.3",		/* 3 */
	"1.5",		/* 4 */
	"2.0",		/* 5 */
	"2.5",		/* 6 */
	"3.0",		/* 7 */
	"3.5",		/* 8 */
	"4.0",		/* 9 */
	"4.5",		/* 10 */
	"5.0",		/* 11 */
	"5.5",		/* 12 */
	"6.0",		/* 13 */
	"7.0",		/* 14 */
	"8.0"		/* 15 */
    };


    bvl = AT91F_DBGU_Printf("TRANS_SPEED", val, AT91C_CSD_TRANS_SPEED_S, 
		AT91C_CSD_TRANS_SPEED_M, " ");

    AT91F_DBGU_Printk( v_speed[ (bvl >> 3) & 0xF ] );
    AT91F_DBGU_Printk( " " );
    AT91F_DBGU_Printk( u_speed[ (bvl & 0x7) ] );
    AT91F_DBGU_Printk( "\n\r" );
 }

 while(1) {
    char ch;
    if(AT91F_DBGU_getch(&ch)) {
	break;
    }
 }

/* RESP1 */
 val = *(resp + 1);
 {
    /* card command classes */
    AT91F_DBGU_Printf( "CCC", val, AT91C_CSD_CCC_S, 
	AT91C_CSD_CCC_M, 0);

    /* max read block length */
    AT91F_DBGU_Printf("RD_B_LEN", val, AT91C_CSD_RD_B_LEN_S, 
	AT91C_CSD_RD_B_LEN_M, 0);

    /* read block partitial */
    AT91F_DBGU_Printf("RD_B_PAR", val, AT91C_CSD_RD_B_PAR_S,
    	AT91C_CSD_RD_B_PAR_M, 0);

    /* write block missallign */
    AT91F_DBGU_Printf("WR_B_MIS", val, AT91C_CSD_WR_B_MIS_S,
	AT91C_CSD_WR_B_MIS_M, 0);

    /* read block missalling */
    AT91F_DBGU_Printf("RD_B_MIS", val, AT91C_CSD_RD_B_MIS_S,
	AT91C_CSD_RD_B_MIS_M, 0);

    /* DSR implemented */
    AT91F_DBGU_Printf("CSD_DSR_I", val, AT91C_CSD_DSR_I_S,
	AT91C_CSD_DSR_I_M, 0);

    /* device size */
    AT91F_DBGU_Printf("CSIZE 11:2", val, AT91C_CSD_CSIZE_H_S,
    	AT91C_CSD_CSIZE_H_M, 0);

    /* (CSIZE + 1) * 2 ** (C_SIZE_MULT+2) * 2**READ_BL_LEN */
 }

 while(1) {
    char ch;
    if(AT91F_DBGU_getch(&ch)) {
	break;
    }
 }

/* RESP2 */
 val = *(resp + 2);
 {
    AT91F_DBGU_Printf("CSIZE 1:0", val, AT91C_CSD_CSIZE_L_S,
	AT91C_CSD_CSIZE_L_M, 0);

    AT91F_DBGU_Printf("RCUR_MIN", val, AT91C_CSD_RCUR_MIN_S,
	AT91C_CSD_RCUR_MIN_M, 0);

    AT91F_DBGU_Printf("RCUR_MAX", val, AT91C_CSD_RCUR_MAX_S,
	AT91C_CSD_RCUR_MAX_M, 0);

    AT91F_DBGU_Printf("VDD_WMIN", val, AT91C_CSD_VDD_WMIN_S,
	AT91C_CSD_VDD_WMIN_M, 0);

    AT91F_DBGU_Printf("VDD_WMAX", val, AT91C_CSD_VDD_WMAX_S,
	AT91C_CSD_VDD_WMAX_M, 0);

    AT91F_DBGU_Printf("C_SIZE_M", val, AT91C_CSD_C_SIZE_M_S,
	AT91C_CSD_C_SIZE_M_M, 0);

    AT91F_DBGU_Printf("SECT_SIZE", val, AT91C_CSD_v22_SECT_SIZE_S,
	AT91C_CSD_v22_SECT_SIZE_M, 0);

    AT91F_DBGU_Printf("ER_GRP_SIZE", val, AT91C_CSD_v22_ER_GRP_SIZE_S,
	AT91C_CSD_v22_ER_GRP_SIZE_M, 0);

    AT91F_DBGU_Printf("WP_GRP_SIZE", val, AT91C_CSD_v22_WP_GRP_SIZE_S,
	AT91C_CSD_v22_WP_GRP_SIZE_M, 0);
    }

 while(1) {
    char ch;
    if(AT91F_DBGU_getch(&ch)) {
	break;
    }
 }

/* RESP3 */
 val = *(resp + 3);
 {
    AT91F_DBGU_Printf("WP_GRP_EN", val, AT91C_CSD_WP_GRP_EN_S,
	AT91C_CSD_WP_GRP_EN_M, 0);

    AT91F_DBGU_Printf("DEF_ECC", val, AT91C_CSD_MMC_DEF_ECC_S,
	AT91C_CSD_MMC_DEF_ECC_M, 0);

    AT91F_DBGU_Printf("R2W_F", val, AT91C_CSD_R2W_F_S,
	AT91C_CSD_R2W_F_M, 0);

    AT91F_DBGU_Printf("WBLEN", val, AT91C_CSD_WBLEN_S,
	AT91C_CSD_WBLEN_M, 0);

    AT91F_DBGU_Printf("WBLOCK_P", val, AT91C_CSD_WBLOCK_P_S,
	AT91C_CSD_WBLOCK_P_M, 0);

    AT91F_DBGU_Printf("FILE_FMT_GRP", val, AT91C_CSD_FILE_FMT_GRP_S,
	AT91C_CSD_FILE_FMT_GRP_M, 0);

    AT91F_DBGU_Printf("COPY", val, AT91C_CSD_COPY_S,
	AT91C_CSD_COPY_M, 0);

    AT91F_DBGU_Printf("PERM_WP", val, AT91C_CSD_PERM_WP_S,
	AT91C_CSD_PERM_WP_M, 0);

    AT91F_DBGU_Printf("TMP_WP", val, AT91C_CSD_TMP_WP_S,
	AT91C_CSD_TMP_WP_M, 0);

    AT91F_DBGU_Printf("FILE_FMT", val, AT91C_CSD_FILE_FMT_S,
	AT91C_CSD_FILE_FMT_M, 0);

    AT91F_DBGU_Printf("ECC", val, AT91C_CSD_MMC_ECC_S,
	AT91C_CSD_MMC_ECC_M, 0);

    AT91F_DBGU_Printf("CRC", val, AT91C_CSD_CRC_S,
	AT91C_CSD_CRC_M, 0);

    AT91F_DBGU_Printf("BIT0", val, AT91C_CSD_BIT0_S,
	AT91C_CSD_BIT0_M, 0);
 }

#endif 														/* MCI_CSD_INFO */

 return(1);
}


//*----------------------------------------------------------------------------
//* \fn    AT91F_Test_MMC
//* \brief Configure MCI for MMC and complete MMC init, then jump to Test Functions
//*----------------------------------------------------------------------------
int AT91F_Test_MMC(void)
{
//////////////////////////////////////////////////////////
//* For MMC Init
//////////////////////////////////////////////////////////
	
#define MMC_CLOCK 100000	

 AT91F_MCI_Configure(AT91C_BASE_MCI,
	/* DTOR_register */
	(AT91C_MCI_DTOMUL_1048576 | AT91C_MCI_DTOCYC)	, 
	/* MR_register */
	 0x8300 | (master_clock / MMC_CLOCK / 2 - 1)	,
	/* bit 15 - PDC mode */ 
	/* bit 10:8 - power save divisor 3 -> 2**3 = 8*/
	/* bit 7:0 - CLKDIV clock divisor 2*(clkdiv + 1) */
	 AT91C_MCI_MMC_SLOTA);	/* SDCR_register */

 /* power save disable */
 AT91C_BASE_MCI->MCI_CR = AT91C_MCI_PWSDIS;	// set disable bit  

 //* Wait MCI Device Ready
 AT91F_MCIDeviceWaitReady(AT91C_MCI_TIMEOUT);

 while(1) {
/* menu definition */
    struct MENU {
	char * name;					/* menu item name */
	unsigned char num; 				/* command number */
	unsigned int cmd; 				/* command register content */
    };
		
    static struct MENU menu[] = {
	{"A. All test"		  ,  0, 0},
    	{"0. GO_IDLE_STATE"	  ,  0, AT91C_MMC_GO_IDLE_STATE_CMD},
    	{"1. SEND_OP_COND"	  ,  1, AT91C_MMC_SEND_OP_COND_CMD},
	{"2. ALL_SEND_CID_CMD"	  ,  2, AT91C_MMC_ALL_SEND_CID_CMD},
	{"3. SET_RELATIVE_ADDR"   ,  3, AT91C_MMC_SET_RELATIVE_ADDR_CMD},
	{"7. SELECT_DESELECT_CARD",  7, AT91C_SEL_DESEL_CARD_CMD},
	{"9. SEND_CSD"		  ,  9, AT91C_SEND_CSD_CMD},
	{"10.SEND_CID"		  , 10, AT91C_SEND_CID_CMD},
	{"13.SEND_STATUS"	  , 13, AT91C_SEND_STATUS_CMD},
	{"15.GO_INACTIVE_STATE"	  , 15, AT91C_GO_INACTIVE_STATE_CMD},
	{"16.SET_BLOCKLEN"	  , 16, AT91C_SET_BLOCKLEN_CMD},
	{"17.READ_SINGLE_BLOCK"	  , 17, AT91C_READ_SINGLE_BLOCK_CMD},
	{"23.SET_BLOCK_COUNT"	  , 23, AT91C_SET_BLOCK_COUNT_CMD},
	{"24.WRITE_BLOCK"	  , 24, AT91C_WRITE_BLOCK_CMD},
	{ 0, 0, 0}
    };


    static int rca = 0xF;		 // MMC RCA
    static unsigned int tab_response[4]; // command response
    static unsigned int r_addr = 0;	 // read address 
    static unsigned int w_addr = 0;	 // write address
    /* read/write word number */
    int rw_pos = 0;		
    unsigned int cmd;			 // command number
    unsigned int arg;			 // command argument
    unsigned int rvl;			 //   
    char ch;
    int i;

    /* print menu */
    for(i = 0; menu[i].name != 0; ++i) { 
	AT91F_DBGU_Printk(menu[i].name);

	if( (i & 1) == 0 ) {
	    int ln = 0;
	    while( *(menu[i].name + ln) != '\0' ) {
		++ln;
	    } 
	
	    while(ln++ < 30) {
    		AT91F_DBGU_Printk(" ");
	    }
	} else {
	   AT91F_DBGU_Printk("\n\r");
	}
    }

    /* read from console command code */

    cmd = 0;
    while(1) {
	if(AT91F_DBGU_getch(&ch)) {
    	    if(ch >= 'a' && ch <= 'z') {
	/* to upper register */
		ch = 'A' + (ch - 'a');
	    }
				
	    if(ch == 'A' || ch == '\n' || ch == '\r') {
	/* all test or Enter -> break */
		break;
	    } else {
		if(ch >= '0' && ch <= '9') {
	    	    char tmp[2];
		    cmd = cmd * 10 + (ch - '0');
		    tmp[0] = ch;
		    tmp[1] = '\0';
	    	    AT91F_DBGU_Printk(tmp);
		}
	    }
	}
    }

    if(ch == 'A') {
    /* original MMC test */		
	break;
    }

    for(i = 1; menu[i].name != 0; ++i) { 
	if((unsigned char) menu[i].num == (unsigned char) cmd) { 
	    AT91F_DBGU_Printk("\n\r===========\n\r");
	    AT91F_DBGU_Printk(menu[i].name);
	    AT91F_DBGU_Printk("\n\r");
	    cmd = menu[i].cmd;
	    { break; }
	}
    }

    /* fill command argument */
    arg = 0;						

    if(cmd == AT91C_MMC_GO_IDLE_STATE_CMD ||
       cmd == AT91C_MMC_ALL_SEND_CID_CMD) {
    } else
    if(cmd == AT91C_MMC_SEND_OP_COND_CMD) {
	arg = AT91C_MMC_HOST_VOLTAGE_RANGE; 
    } else 
    if(cmd == AT91C_MMC_SET_RELATIVE_ADDR_CMD) {
	rca++;
	arg = (((unsigned int)rca) << 16);
	AT91F_DBGU_Printv("Set RCA", rca, "\n\r");
    } else 
    if(
        (cmd == AT91C_SEL_DESEL_CARD_CMD) ||	/* select/deselect card  */
 	(cmd == AT91C_SEND_CSD_CMD)       ||	/* send Card Spec Data register */
	(cmd == AT91C_SEND_CID_CMD)	      ||	/* send Card Ident-on Data reg */
	(cmd == AT91C_SEND_STATUS_CMD)	  ||	/* send status command */
	(cmd == AT91C_GO_INACTIVE_STATE_CMD)	/* go inactive state */
    ) {
	arg = (((unsigned int)rca) << 16);
    } else 
    if(cmd == AT91C_SET_BLOCKLEN_CMD) {
/* set block length */
	arg = MCI_Device.card[0].Max_Read_DataBlock_Length;
	if(arg == 0) {
    	    arg = 512;
	}

	AT91F_DBGU_Printv("Block Len", arg, 0);
    } else 
    if(cmd == AT91C_SET_BLOCK_COUNT_CMD) {
/* set block count command */
	arg = 2;
	AT91F_DBGU_Printv("Block count", arg, 0);
    } else
    if(cmd == AT91C_READ_SINGLE_BLOCK_CMD) {
			
	AT91F_DBGU_Printv("Read block, addr ", r_addr, 0);

	/* command argument - read address */
	arg = r_addr;
	/* clear PDC mode bit, set block length */
	AT91C_BASE_MCI->MCI_MR &= (~AT91C_MCI_PDCMODE);
	AT91C_BASE_MCI->MCI_MR |=
	    (MCI_Device.card[0].Max_Read_DataBlock_Length << 16);

	for(rw_pos = 0; rw_pos < sizeof(Buffer); ++rw_pos) {
	    Buffer[rw_pos] = 0;
	}

	rw_pos = 0;						/* buffer position */
    } else  
    if(cmd == AT91C_WRITE_BLOCK_CMD) {
	AT91F_DBGU_Printv("Write block, addr ", w_addr, 0);

	/* command argument - read address */
	arg = w_addr;
	
	/* clear PDC mode bit, set block length */
	AT91C_BASE_MCI->MCI_MR &= (~AT91C_MCI_PDCMODE);
	AT91C_BASE_MCI->MCI_MR |=
	    (MCI_Device.card[0].Max_Read_DataBlock_Length << 16);

	/* fill write buffer */	
	for(rw_pos = 0; rw_pos < sizeof(Buffer); ++rw_pos) {
	    Buffer[rw_pos] = (unsigned char) (rw_pos);
	}
	
	*((unsigned int *)Buffer + 0) = w_addr; 	/* write address */
	*((unsigned int *)Buffer + 1) = 0; 
	   rw_pos = 0;
    } else  {
	AT91F_DBGU_Printk("Unknown Command");
	continue;
    }


    /* send command */
    rvl = AT91F_MCI_SendCommand(& MCI_Device, cmd, arg, tab_response);
		
    if( 
       (cmd != AT91C_READ_SINGLE_BLOCK_CMD) &&
       (cmd != AT91C_WRITE_BLOCK_CMD)
    ) {
	AT91F_MCI_Resp_Info(tab_response);
    }

    if(cmd == AT91C_MMC_SEND_OP_COND_CMD) {
	AT91F_MCI_OCR_Info (tab_response);
    } else 
    if(
       (cmd == AT91C_MMC_ALL_SEND_CID_CMD) ||
       (cmd == AT91C_SEND_CID_CMD)				
    ) {
	AT91F_MCI_CID_Info (tab_response);
    } else 
    if(cmd == AT91C_SEND_CSD_CMD) {
/* send card specific data register */
	AT91F_MCI_CSD_Info (tab_response);

	if(rvl == AT91C_CMD_SEND_OK) {
	    AT91S_MciCard * ci;				
	    unsigned int val1, val2, val3;
	    unsigned int mult;
	    unsigned int csize_l;
	    unsigned int csize_h;

	    ci = MCI_Device.card;
	    val1 = tab_response[1];
	    val2 = tab_response[2];
	    val3 = tab_response[3];
	
	    /* resp1 */
	    ci->Max_Read_DataBlock_Length = 1 << 
		((val1 >> AT91C_CSD_RD_B_LEN_S) & AT91C_CSD_RD_B_LEN_M);						
	
	    ci->Read_Partial = 
		((val1 >> AT91C_CSD_RD_B_PAR_S) & AT91C_CSD_RD_B_PAR_M);
	    ci->Read_Block_Misalignment = 
		((val1 >> AT91C_CSD_RD_B_MIS_S) & AT91C_CSD_RD_B_MIS_M);
	    ci->Write_Block_Misalignment = 
		((val1 >> AT91C_CSD_WR_B_MIS_S) & AT91C_CSD_WR_B_MIS_M);

	    /* resp3 */
	    ci->Max_Write_DataBlock_Length =	1 << 
	 	((val3 >> AT91C_CSD_WBLEN_S) & AT91C_CSD_WBLEN_M);

	    ci->Write_Partial = 
		((val3 >> AT91C_CSD_WBLOCK_P_S) & AT91C_CSD_WBLOCK_P_M);

	    ci->Sector_Size = 1 + 
		((val2 >> AT91C_CSD_v22_SECT_SIZE_S) & AT91C_CSD_v22_SECT_SIZE_M);

	    // None in MMC specification version 2.2
	    ci->Erase_Block_Enable = 0;

	    /* c_size_mult */
	    mult = 1 << ( 
		((val2 >> AT91C_CSD_C_SIZE_M_S) & AT91C_CSD_C_SIZE_M_M) + 2);

	    // 2low bits
	    csize_l = 
		((val2 >> AT91C_CSD_CSIZE_L_S) & AT91C_CSD_CSIZE_L_M);

	    csize_h = 
		((val1 >> AT91C_CSD_CSIZE_H_S) & AT91C_CSD_CSIZE_H_M);


	    ci->Memory_Capacity = 
		ci->Max_Read_DataBlock_Length * 
		mult * ( (csize_h << 2) | csize_l );
		
#ifdef MCI_CSD_DEBUG
   	    AT91F_DBGU_Printv("RCA", ci->Relative_Card_Address,0);
	    AT91F_DBGU_Printv("Read Bl Length ", ci->Max_Read_DataBlock_Length,0);
	    AT91F_DBGU_Printv("Write Bl Length", ci->Max_Write_DataBlock_Length,0);
	    AT91F_DBGU_Printv("Read bl partial", ci->Read_Partial,0);
	    AT91F_DBGU_Printv("Write blpartial", ci->Write_Partial,0);
	    AT91F_DBGU_Printv("Read Bl Misal  ", ci->Read_Block_Misalignment,0);
	    AT91F_DBGU_Printv("Write Bl Misal ", ci->Write_Block_Misalignment,0);
	    AT91F_DBGU_Printv("Erase bl enable", ci->Erase_Block_Enable,0);
	    AT91F_DBGU_Printv("Sector size    ", ci->Sector_Size,0);
	    AT91F_DBGU_Printv("Memory Capacity", ci->Memory_Capacity,0);
#endif 												/* MCI_CSD_DEBUG */
	}
    } else
    if(
	(cmd == AT91C_MMC_SET_RELATIVE_ADDR_CMD) ||
	(cmd == AT91C_SEL_DESEL_CARD_CMD)        ||
	(cmd == AT91C_SEND_STATUS_CMD)		 ||
	(cmd == AT91C_SET_BLOCKLEN_CMD)		 || /* set block length  */
	(cmd == AT91C_SET_BLOCK_COUNT_CMD)	    /* set block count */
    ) {
    /* print R1 response */	
	if(cmd == AT91C_MMC_SET_RELATIVE_ADDR_CMD) {
	    if(rvl == AT91C_CMD_SEND_OK) {
		MCI_Device.card[0].Relative_Card_Address = rca;
	    }
	}

        AT91F_MCI_R1_Info  (tab_response);
    } else  	
    if(cmd == AT91C_READ_SINGLE_BLOCK_CMD) {
	int len;
	len = MCI_Device.card[0].Max_Read_DataBlock_Length;
	if(len > 512) {
	    len = 512;
	}
	len /= 4;

	rw_pos = 0;
	while(rw_pos < len) {
	    unsigned volatile int rval = AT91C_BASE_MCI->MCI_SR;
 	    if( (rval & AT91C_MCI_RXRDY) != 0 ) {
		rval = AT91C_BASE_MCI->MCI_RDR;
		*( ((unsigned int *) Buffer) + rw_pos ) = rval;
		 rw_pos++;
 	    }
	}

	AT91F_DBGU_Dump(Buffer, r_addr, len * 4); 
	r_addr += len * 4;

	AT91F_MCI_R1_Info  (tab_response);
    } else 
    if(cmd == AT91C_WRITE_BLOCK_CMD) {
    /* write single block */
	int len;
	len = MCI_Device.card[0].Max_Read_DataBlock_Length;
	if(len > 512) {
    	    len = 512;
	}
	len /= 4;
		
	rw_pos = 0;
	while(rw_pos < len) {
	    unsigned volatile int rval = AT91C_BASE_MCI->MCI_SR;
 	    if( (rval & AT91C_MCI_TXRDY) != 0 ) {
		AT91C_BASE_MCI->MCI_TDR = 
		 *( ((unsigned int *) Buffer) + rw_pos );
		rw_pos++;
 	    }
	}
	
	AT91F_DBGU_Dump(Buffer, w_addr, len * 4); 
	w_addr += len * 4;

	AT91F_MCI_R1_Info  (tab_response);
    } else 
    if(cmd == AT91C_GO_INACTIVE_STATE_CMD) {
    }else  {
    }

    if(rvl == AT91C_CMD_SEND_OK) {
	AT91F_DBGU_Printk("Ok\n\r");
    } else {
	AT91F_DBGU_Printk("Fail\n\r");
    }

	;
 }

#ifdef MCI_DEBUG
 AT91F_DBGU_Printk("\n\rMMC Init ");
#endif 													/* MCI_DEBUG */
 if(AT91F_MCI_MMC_Init(&MCI_Device) != AT91C_INIT_OK) {
#ifdef MCI_DEBUG
    AT91F_DBGU_Printk("Fail\n\r");
#endif 													/* MCI_DEBUG */
    return 0;
 }

 // Select MMC Card n1
#ifdef MCI_DEBUG
 AT91F_DBGU_Printk("\n\rMMC Select Card ");
#endif 													/* MCI_DEBUG */
 if(AT91F_MCI_MMC_SelectCard(&MCI_Device,AT91C_FIRST_RCA) != AT91C_CARD_SELECTED_OK) {
#ifdef MCI_DEBUG
    AT91F_DBGU_Printk("Fail\n\r");
#endif 													/* MCI_DEBUG */
    return 0;
 }

// Set Block Length
 {
    int len;
    len = MCI_Device.card[0].Max_Read_DataBlock_Length;
    if(len > MCI_Device.card[0].Max_Write_DataBlock_Length) {
	len = MCI_Device.card[0].Max_Write_DataBlock_Length;
    }

#ifdef MCI_DEBUG
    AT91F_DBGU_Printk("\n\rMMC Set Block Len ");
#endif 													/* MCI_DEBUG */
    if (AT91F_MCI_SetBlocklength(&MCI_Device,len) != AT91C_CMD_SEND_OK) {
#ifdef MCI_DEBUG
	AT91F_DBGU_Printk("Fail\n\r");
#endif 													/* MCI_DEBUG */
	return 0;
    }
 }

 AT91F_DBGU_Printk("\n\rMMC Initialisation Successful:\n\r");

// Enter Main Tests	
 return(AT91F_Test());
}

#ifdef SDCARD_TEST 
//*----------------------------------------------------------------------------
//* \fn    AT91F_Test_SDCard
//* \brief Configure MCI for SDCard and complete SDCard init, then jump to Test Functions
//*----------------------------------------------------------------------------
int AT91F_Test_SDCard(void)
{
//////////////////////////////////////////////////////////
//* For SDCard Init
//////////////////////////////////////////////////////////

 AT91F_MCI_Configure(AT91C_BASE_MCI,
    (AT91C_MCI_DTOMUL_1048576 | AT91C_MCI_DTOCYC)	, 
    // 15MHz for MCK = 60MHz (CLKDIV = 1)
    (AT91C_MCI_MR_PDCMODE | 1),			
    AT91C_MCI_SDCARD_4BITS_SLOTA);
	
 if(AT91F_MCI_SDCard_Init(&MCI_Device) != AT91C_INIT_OK)
    return 0;

 AT91F_DBGU_Printk("\n\rSDCard Initialisation Successful: Enter Test\n\r");

// Enter Main Tests	
    return(AT91F_Test());
}
#endif 															/* SDCARD_TEST */

//*----------------------------------------------------------------------------
//* \fn    AT91F_MCI_Handler
//* \brief MCI Handler
//*----------------------------------------------------------------------------
void AT91F_MCI_Handler(void)
{
 int status;

 status = ( AT91C_BASE_MCI->MCI_SR & AT91C_BASE_MCI->MCI_IMR );

 AT91F_MCI_Device_Handler(&MCI_Device,status);
}

//*----------------------------------------------------------------------------
//* \fn    main
//* \brief main function
//*----------------------------------------------------------------------------
int main(void)
{
#ifdef SDCARD_TEST 
 char caractere;
#endif 														/* SDCARD_TEST */
	
 AT91F_DBGU_Printk("AT91RM9200 MCI Device Test. ");
 AT91F_DBGU_Printl(master_clock);
 AT91F_DBGU_Printk("Hz");
///////////////////////////////////////////////////////////////////////////////////////////
//  MCI Init : common to MMC and SDCard
///////////////////////////////////////////////////////////////////////////////////////////

 AT91F_DBGU_Printk("\n\rInit MCI Interface\n\r");

#ifndef EVM9200
// Set up PIO SDC_TYPE to switch on MMC/SDCard and not DataFlash Card
 AT91F_PIO_CfgOutput(AT91C_BASE_PIOB,AT91C_PIO_PB7);
 AT91F_PIO_SetOutput(AT91C_BASE_PIOB,AT91C_PIO_PB7);
#endif 														/* not EVM9200 */

// Init MCI for MMC and SDCard interface
 AT91F_MCI_CfgPIO();	
 AT91F_MCI_CfgPMC();

#ifdef EVM9200
/* */	
 AT91F_PIO_CfgPullup(
    AT91C_BASE_PIOA, 			// PIO controller base address
	(
	 ((unsigned int) AT91C_PA28_MCCDA   ) |
	 ((unsigned int) AT91C_PA29_MCDA0   ))
    );
#endif 														/* EVM9200 */
/* disable power save mode */
 AT91C_BASE_MCI->MCI_CR = AT91C_MCI_PWSDIS;

 AT91F_PDC_Open(AT91C_BASE_PDC_MCI);

 // Disable all the interrupts
 AT91C_BASE_MCI->MCI_IDR = 0xFFFFFFFF;

 // Init MCI Device Structures
 AT91F_CfgDevice();

 // Configure MCI interrupt 
 AT91F_AIC_ConfigureIt(AT91C_BASE_AIC,
     AT91C_ID_MCI,
     AT91C_AIC_PRIOR_HIGHEST,
     AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE,
     AT91F_ASM_MCI_Handler);

 // Enable MCI interrupt
 AT91F_AIC_EnableIt(AT91C_BASE_AIC,AT91C_ID_MCI);

///////////////////////////////////////////////////////////////////////////////////////////
//  Enter Test Menu
///////////////////////////////////////////////////////////////////////////////////////////

 // Enable Receiver
 AT91F_US_EnableRx((AT91PS_USART) AT91C_BASE_DBGU);

 while(1) {
#ifdef SDCARD_TEST 
    AT91F_DBGU_Printk("1:Test MMC\n\r");
    AT91F_DBGU_Printk("2:Test SDCard\n\r");

    // Wait for User Choice : 1 <=> Enter Test MMC , 2 <=> Enter Test SDCard
    while(1) {
	if(AT91F_DBGU_getch(& caractere)) {
    	    break;
	}
    }

    switch(caractere) {
	case '1':
	    AT91F_DBGU_Printk("\n\rEnter Test MMC\n\r");
	    if(AT91F_Test_MMC() == 1) {
		AT91F_DBGU_Printk("Ok\n\r");
	    } else {
		AT91F_DBGU_Printk("Fail\n\r");
	    }
	    break;
			
	case '2':	
	    AT91F_DBGU_Printk("\n\rEnter Test SDCard\n\r");

	    if(AT91F_Test_SDCard() == 1) {
		AT91F_DBGU_Printk("Ok\n\r");
	    } else {
		AT91F_DBGU_Printk("Fail\n\r");
	    }	
	    break;
			
	default:
	    AT91F_DBGU_Printk("\n\rBad choice, Retry please\n\r");
	    break;
    }
#else 		
    if(AT91F_Test_MMC() == 1) {
        AT91F_DBGU_Printk("Ok\n\r");
    } else {
        AT91F_DBGU_Printk("Fail\n\r");
    }
#endif 													/* SDCARD_TEST */
 }
}
