//*---------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*---------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*---------------------------------------------------------------------------
//* File Name           : AT91C_MCI_Device.h
//* Object              : Data Flash Atmel Description File
//* Translator          :
//*
//* 1.0 26/11/02 FB		: Creation
//*---------------------------------------------------------------------------

/* modified in 2004-2005 by Dmitriy Cherkashin */
/* dch@ucrouter.ru,  www.ucrouter.ru */
/* based on ATMEL AT91RM9200-BasicMCIDevice code */

#ifndef AT91C_MCI_Device_h
#define AT91C_MCI_Device_h

#include "AT91RM9200.h"

#ifdef __GNUC__
#include "proto.h"			
/* *****************************************************************************
                SOFTWARE API FOR MCI
   ***************************************************************************** */
//* Classic MCI Mode Register Configuration with PDC mode enabled and MCK = MCI Clock
#define AT91C_MCI_MR_PDCMODE	(AT91C_MCI_CLKDIV |\
                                AT91C_MCI_PWSDIV |\
                                (AT91C_MCI_PWSDIV<<1) |\
                                AT91C_MCI_PDCMODE)

//* Classic MCI Data Timeout Register Configuration with 1048576 MCK cycles between 2 data transfer
#define AT91C_MCI_DTOR_1MEGA_CYCLES	(AT91C_MCI_DTOCYC | AT91C_MCI_DTOMUL)

//* Classic MCI SDCard Register Configuration with 1-bit data bus on slot A
#define AT91C_MCI_MMC_SLOTA	(AT91C_MCI_SCDSEL & 0x0)

//* Classic MCI SDCard Register Configuration with 1-bit data bus on slot B
#define AT91C_MCI_MMC_SLOTB	(AT91C_MCI_SCDSEL)

//* Classic MCI SDCard Register Configuration with 4-bit data bus on slot A
#define AT91C_MCI_SDCARD_4BITS_SLOTA	( (AT91C_MCI_SCDSEL & 0x0) | AT91C_MCI_SCDBUS )

//* Classic MCI SDCard Register Configuration with 4-bit data bus on slot B
#define AT91C_MCI_SDCARD_4BITS_SLOTB	(AT91C_MCI_SCDSEL | AT91C_MCI_SCDBUS)
#else 
#include "lib_AT91RM9200.h"
#endif							/* __GNUC__ */

#undef SDCARD_TEST

typedef unsigned int AT91S_MCIDeviceStatus;

/////////////////////////////////////////////////////////////////////////////////////////////////////
/* Card_Inserted field */
#define AT91C_CARD_REMOVED		0
#define AT91C_MMC_CARD_INSERTED		1
#define AT91C_SD_CARD_INSERTED		2

#define AT91C_NO_ARGUMENT		0x0

#define AT91C_FIRST_RCA			0xCAFE
#define AT91C_MAX_MCI_CARDS		10

/* Driver State */
#define AT91C_MCI_IDLE       		0x0
#define AT91C_MCI_TIMEOUT_ERROR		0x1
#define AT91C_MCI_RX_SINGLE_BLOCK	0x2
#define AT91C_MCI_RX_MULTIPLE_BLOCK	0x3
#define AT91C_MCI_RX_STREAM		0x4
#define AT91C_MCI_TX_SINGLE_BLOCK	0x5
#define AT91C_MCI_TX_MULTIPLE_BLOCK	0x6
#define AT91C_MCI_TX_STREAM 		0x7

/* TimeOut */
#define AT91C_TIMEOUT_CMDRDY		30

/////////////////////////////////////////////////////////////////////////////////////////////////////
// MMC & SDCard Control Structure 
/////////////////////////////////////////////////////////////////////////////////////////////////////

/* MultiMedia Card definition */ 
typedef struct _AT91S_MciCard {
 unsigned char	Card_Inserted;		    // (0=AT91C_CARD_REMOVED) (1=AT91C_MMC_CARD_INSERTED) (2=AT91C_SD_CARD_INSERTED)
 unsigned int 	Relative_Card_Address;	    // RCA
 unsigned int 	Max_Read_DataBlock_Length;  // 2^(READ_BL_LEN) in CSD 
 unsigned int 	Max_Write_DataBlock_Length; // 2^(WRITE_BL_LEN) in CSD
 unsigned char	Read_Partial;		    // READ_BL_PARTIAL
 unsigned char	Write_Partial;		    // WRITE_BL_PARTIAL
 unsigned char	Erase_Block_Enable;	    // ERASE_BLK_EN
 unsigned char	Read_Block_Misalignment;    // READ_BLK_MISALIGN
 unsigned char	Write_Block_Misalignment;   // WRITE_BLK_MISALIGN
 unsigned char	Sector_Size;		    // SECTOR_SIZE
 unsigned int	Memory_Capacity;	    // Size in bits of the device
} AT91S_MciCard;

/* MMC bus definition */
typedef struct _AT91S_MciDevice {
 volatile unsigned char	state;
 unsigned char	SDCard_bus_width;
 unsigned int	card_num;		    // Size in bits of the device
 AT91S_MciCard	card[AT91C_MAX_MCI_CARDS];  // MMC definition 
}AT91S_MciDevice, *AT91PS_MciDevice;

/////////////////////////////////////////////////////////////////////////////////////////////////////
// MCI_CMD Register Value 
/////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Class 0 & 1 commands: Basic commands and Read Stream commands
//////////////////////////////////////////////////////////////////////////////
#undef OPD_ENADBLE

/* CMD0 - GO_IDLE_STATE - reset all cards to idle state */
#define AT91C_GO_IDLE_STATE_CMD								\
	(0 	| AT91C_MCI_TRCMD_NO 	| AT91C_MCI_SPCMD_NONE )

#ifdef OPD_ENABLE
#define AT91C_MMC_GO_IDLE_STATE_CMD \
	(AT91C_GO_IDLE_STATE_CMD | AT91C_MCI_OPDCMD)
#else 							/* OPD_ENABLE */
#define AT91C_MMC_GO_IDLE_STATE_CMD \
	(AT91C_GO_IDLE_STATE_CMD)
#endif 							

/* CMD1 - SEND_OP_COND - ask all cards in idle state to send their operation */
/* conditions reg contents in the response on the CMD line */

#define AT91C_SEND_OP_COND_CMD \
	(1	| AT91C_MCI_TRCMD_NO 	| AT91C_MCI_SPCMD_NONE | \
		  AT91C_MCI_RSPTYP_48	| AT91C_MCI_MAXLAT \
)

#ifdef OPD_ENABLE
#define AT91C_MMC_SEND_OP_COND_CMD \
	(AT91C_SEND_OP_COND_CMD	 | AT91C_MCI_OPDCMD)
#else 
#define AT91C_MMC_SEND_OP_COND_CMD \
	(AT91C_SEND_OP_COND_CMD)
#endif 							/* OPD_ENABLE */

/* CMD2 asks all cards to send their CID on the CMD line */
#define AT91C_ALL_SEND_CID_CMD \
	(2	| AT91C_MCI_TRCMD_NO 	| AT91C_MCI_SPCMD_NONE	| \
		  AT91C_MCI_RSPTYP_136  | AT91C_MCI_MAXLAT)

#ifdef OPD_ENABLE
#define AT91C_MMC_ALL_SEND_CID_CMD \
	(AT91C_ALL_SEND_CID_CMD | AT91C_MCI_OPDCMD)
#else 
#define AT91C_MMC_ALL_SEND_CID_CMD \
	(AT91C_ALL_SEND_CID_CMD)
#endif 							/* OPD_ENABLE */

/* CMD3 */
#define AT91C_SET_RELATIVE_ADDR_CMD \
	(3	| AT91C_MCI_TRCMD_NO 	| AT91C_MCI_SPCMD_NONE	| \
		  AT91C_MCI_RSPTYP_48	| AT91C_MCI_MAXLAT )

#ifdef OPD_ENABLE
#define AT91C_MMC_SET_RELATIVE_ADDR_CMD	\
		(AT91C_SET_RELATIVE_ADDR_CMD | AT91C_MCI_OPDCMD)
#else 
#define AT91C_MMC_SET_RELATIVE_ADDR_CMD	\
		(AT91C_SET_RELATIVE_ADDR_CMD)
#endif 							/* OPD_ENABLE */

#define AT91C_SET_DSR_CMD \
	(4	| AT91C_MCI_TRCMD_NO 	| AT91C_MCI_SPCMD_NONE	| \
		  AT91C_MCI_RSPTYP_NO	| AT91C_MCI_MAXLAT )	// no tested

#define AT91C_SEL_DESEL_CARD_CMD \
	(7	| AT91C_MCI_TRCMD_NO 	| AT91C_MCI_SPCMD_NONE	| \
		  AT91C_MCI_RSPTYP_48  	| AT91C_MCI_MAXLAT )

#define AT91C_SEND_CSD_CMD \
	(9	| AT91C_MCI_TRCMD_NO 	| AT91C_MCI_SPCMD_NONE	| \
		  AT91C_MCI_RSPTYP_136  | AT91C_MCI_MAXLAT )

#define AT91C_SEND_CID_CMD \
	(10	| AT91C_MCI_TRCMD_NO 	| AT91C_MCI_SPCMD_NONE	| \
	      AT91C_MCI_RSPTYP_136 	| AT91C_MCI_MAXLAT )

#define AT91C_MMC_READ_DAT_UNTIL_STOP_CMD \
	(11	| AT91C_MCI_TRTYP_STREAM| AT91C_MCI_SPCMD_NONE	| \
		  AT91C_MCI_RSPTYP_48	| AT91C_MCI_TRDIR	| \
		  AT91C_MCI_TRCMD_START | AT91C_MCI_MAXLAT )

#define AT91C_STOP_TRANSMISSION_CMD \
	(12	| AT91C_MCI_TRCMD_STOP 	| AT91C_MCI_SPCMD_NONE	| \
		  AT91C_MCI_RSPTYP_48 	| AT91C_MCI_MAXLAT )

#define AT91C_STOP_TRANSMISSION_SYNC_CMD \
	(12	| AT91C_MCI_TRCMD_STOP 	| AT91C_MCI_SPCMD_SYNC	| \
		  AT91C_MCI_RSPTYP_48 	| AT91C_MCI_MAXLAT )

#define AT91C_SEND_STATUS_CMD \
	(13	| AT91C_MCI_TRCMD_NO 	| AT91C_MCI_SPCMD_NONE	| \
		  AT91C_MCI_RSPTYP_48 	| AT91C_MCI_MAXLAT )
#define AT91C_GO_INACTIVE_STATE_CMD \
	(15	| AT91C_MCI_RSPTYP_NO )

//*------------------------------------------------
//* Class 2 commands: Block oriented Read commands
//*------------------------------------------------

#define AT91C_SET_BLOCKLEN_CMD \
	(16 	| AT91C_MCI_TRCMD_NO 	| AT91C_MCI_SPCMD_NONE	| \
		  AT91C_MCI_RSPTYP_48	| AT91C_MCI_MAXLAT )

#define AT91C_READ_SINGLE_BLOCK_CMD \
	(17 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 	| \
		  AT91C_MCI_TRCMD_START	| AT91C_MCI_TRTYP_BLOCK	| \
		  AT91C_MCI_TRDIR	| AT91C_MCI_MAXLAT)

#define AT91C_READ_MULTIPLE_BLOCK_CMD \
	(18 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 	| \
		  AT91C_MCI_TRCMD_START	| AT91C_MCI_TRTYP_MULTIPLE | \
		  AT91C_MCI_TRDIR	| AT91C_MCI_MAXLAT)

//*--------------------------------------------
//* Class 3 commands: Sequential write commands
//*--------------------------------------------

#define AT91C_MMC_WRITE_DAT_UNTIL_STOP_CMD \
	(20 	| AT91C_MCI_TRTYP_STREAM| AT91C_MCI_SPCMD_NONE	| \
		  AT91C_MCI_RSPTYP_48 & ~(AT91C_MCI_TRDIR) | \
		  AT91C_MCI_TRCMD_START | AT91C_MCI_MAXLAT )	// MMC

//*------------------------------------------------
//* Class 4 commands: Block oriented write commands
//*------------------------------------------------

/* set block count command */
#define AT91C_SET_BLOCK_COUNT_CMD	\
(23 | AT91C_MCI_TRCMD_NO 	|	\
      AT91C_MCI_SPCMD_NONE	| 	\
      AT91C_MCI_RSPTYP_48	| 	\
      AT91C_MCI_MAXLAT)

#define AT91C_WRITE_BLOCK_CMD \
	(24	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
		  AT91C_MCI_TRCMD_START	| (AT91C_MCI_TRTYP_BLOCK \
	     &  ~(AT91C_MCI_TRDIR))	| AT91C_MCI_MAXLAT)

#define AT91C_WRITE_MULTIPLE_BLOCK_CMD \
	(25 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
		  AT91C_MCI_TRCMD_START	| (AT91C_MCI_TRTYP_MULTIPLE \
	     &  ~(AT91C_MCI_TRDIR)) 	| AT91C_MCI_MAXLAT)

#define AT91C_PROGRAM_CSD_CMD \
	(27 	| AT91C_MCI_RSPTYP_48 )

//*----------------------------------------
//* Class 6 commands: Group Write protect
//*----------------------------------------

#define AT91C_SET_WRITE_PROT_CMD \
	(28	| AT91C_MCI_RSPTYP_48 )

#define AT91C_CLR_WRITE_PROT_CMD \
	(29	| AT91C_MCI_RSPTYP_48 )

#define AT91C_SEND_WRITE_PROT_CMD \
	(30	| AT91C_MCI_RSPTYP_48 )

//*----------------------------------------
//* Class 5 commands: Erase commands
//*----------------------------------------

#define AT91C_TAG_SECTOR_START_CMD \
	(32 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48	| \
		  AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT)

#define AT91C_TAG_SECTOR_END_CMD \
	(33 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48	| \
		  AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT)

#define AT91C_MMC_UNTAG_SECTOR_CMD \
	(34 	| AT91C_MCI_RSPTYP_48 )

#define AT91C_MMC_TAG_ERASE_GROUP_START_CMD \
	(35 	| AT91C_MCI_RSPTYP_48 )

#define AT91C_MMC_TAG_ERASE_GROUP_END_CMD \
	(36 	| AT91C_MCI_RSPTYP_48 )

#define AT91C_MMC_UNTAG_ERASE_GROUP_CMD	\
	(37 	| AT91C_MCI_RSPTYP_48 )

#define AT91C_ERASE_CMD	\
	(38 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48	| \
		  AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT )

//*----------------------------------------
//* Class 7 commands: Lock commands
//*----------------------------------------

#define AT91C_LOCK_UNLOCK \
	(42 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48	| \
		  AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT)	// no tested

//*-----------------------------------------------
// Class 8 commands: Application specific commands
//*-----------------------------------------------

#define AT91C_APP_CMD \
	(55 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
		  AT91C_MCI_TRCMD_NO | AT91C_MCI_MAXLAT)

#define AT91C_GEN_CMD \
	(56	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
		  AT91C_MCI_TRCMD_NO | AT91C_MCI_MAXLAT)	// no tested

#define AT91C_SDCARD_SET_BUS_WIDTH_CMD \
	(6 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
		  AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT)

#define AT91C_SDCARD_STATUS_CMD \
    	(13 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
	          AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT)

#define AT91C_SDCARD_SEND_NUM_WR_BLOCKS_CMD \
	(22 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
	          AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT)

#define AT91C_SDCARD_SET_WR_BLK_ERASE_COUNT_CMD	\
	(23 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
	          AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT)

#define AT91C_SDCARD_APP_OP_COND_CMD \
	(41 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
	          AT91C_MCI_TRCMD_NO )

#define AT91C_SDCARD_SET_CLR_CARD_DETECT_CMD \
	(42 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
	          AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT)

#define AT91C_SDCARD_SEND_SCR_CMD \
	(51     | AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
	          AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT)

#define AT91C_SDCARD_APP_ALL_CMD \
	(AT91C_SDCARD_SET_BUS_WIDTH_CMD +\
	 AT91C_SDCARD_STATUS_CMD +\
	 AT91C_SDCARD_SEND_NUM_WR_BLOCKS_CMD +\
	 AT91C_SDCARD_SET_WR_BLK_ERASE_COUNT_CMD +\
	 AT91C_SDCARD_APP_OP_COND_CMD +\
	 AT91C_SDCARD_SET_CLR_CARD_DETECT_CMD +\
	 AT91C_SDCARD_SEND_SCR_CMD)

//*----------------------------------------
//* Class 9 commands: IO Mode commands
//*----------------------------------------

#define AT91C_MMC_FAST_IO_CMD \
	(39	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
		  AT91C_MCI_MAXLAT)

#define AT91C_MMC_GO_IRQ_STATE_CMD \
	(40 	| AT91C_MCI_SPCMD_NONE	| AT91C_MCI_RSPTYP_48 | \
		  AT91C_MCI_TRCMD_NO	| AT91C_MCI_MAXLAT)

/////////////////////////////////////////////////////////////////////////////////////////////////////
// Functions returnals
/////////////////////////////////////////////////////////////////////////////////////////////////////
#define AT91C_CMD_SEND_OK	   0 // Command ok
#define AT91C_CMD_SEND_ERROR      -1 // Command failed
#define AT91C_INIT_OK		   2 // Init Successfull
#define AT91C_INIT_ERROR	   3 // Init Failed
#define AT91C_READ_OK	  	   4 // Read Successfull
#define AT91C_READ_ERROR	   5 // Read Failed
#define AT91C_WRITE_OK		   6 // Write Successfull
#define AT91C_WRITE_ERROR	   7 // Write Failed
#define AT91C_ERASE_OK	 	   8 // Erase Successfull
#define AT91C_ERASE_ERROR	   9 // Erase Failed
#define AT91C_CARD_SELECTED_OK    10 // Card Selection Successfull
#define AT91C_CARD_SELECTED_ERROR 11 // Card Selection Failed

/////////////////////////////////////////////////////////////////////////////////////////////////////
// MCI_SR Errors
/////////////////////////////////////////////////////////////////////////////////////////////////////
#define AT91C_MCI_SR_ERROR \
	(AT91C_MCI_UNRE  | \
	 AT91C_MCI_OVRE  | \
	 AT91C_MCI_DTOE  | \
	 AT91C_MCI_DCRCE | \
	 AT91C_MCI_RTOE  | \
	 AT91C_MCI_RENDE | \
	 AT91C_MCI_RCRCE | \
	 AT91C_MCI_RDIRE | \
	 AT91C_MCI_RINDE)

////////////////////////////////////////////////////////////////////////////////////////////////////
// OCR Register
////////////////////////////////////////////////////////////////////////////////////////////////////
#define AT91C_VDD_145_150 (1 <<  0)	/* 1.45 - 1.50 */
#define AT91C_VDD_150_155 (1 <<  1)	/* 1.50 - 1.55 */
#define AT91C_VDD_155_160 (1 <<  2)	/* 1.55 - 1.60 */
#define AT91C_VDD_160_165 (1 <<  3)	/* 1.60 - 1.65 */
#define AT91C_VDD_165_17  (1 <<  4)	/* 1.65 - 1.70 */
#define AT91C_VDD_17_18	  (1 <<  5)	/* 1.70 - 1.80 */
#define AT91C_VDD_18_19	  (1 <<  6)	/* 1.80 - 1.90 */
#define AT91C_VDD_19_20	  (1 <<  7)	/* 1.90 - 2.00 */
#define AT91C_VDD_20_21	  (1 <<  8)	/* 2.00 - 2.10 */
#define AT91C_VDD_21_22	  (1 <<  9)	/* 2.10 - 2.20 */
#define AT91C_VDD_22_23	  (1 << 10)	/* 2.20 - 2.30 */
#define AT91C_VDD_23_24	  (1 << 11)	/* 2.30 - 2.40 */
#define AT91C_VDD_24_25	  (1 << 12)	/* 2.40 - 2.50 */
#define AT91C_VDD_25_26	  (1 << 13)	/* 2.50 - 2.60 */
#define AT91C_VDD_26_27	  (1 << 14)	/* 2.60 - 2.70 */
#define AT91C_VDD_27_28	  (1 << 15)	/* 2.70 - 2.80 */
#define AT91C_VDD_28_29	  (1 << 16)	/* 2.80 - 2.90 */
#define AT91C_VDD_29_30	  (1 << 17)	/* 2.90 - 3.00 */
#define AT91C_VDD_30_31	  (1 << 18)	/* 3.00 - 3.10 */
#define AT91C_VDD_31_32	  (1 << 19)	/* 3.10 - 3.20 */
#define AT91C_VDD_32_33	  (1 << 20)	/* 3.20 - 3.30 */
#define AT91C_VDD_33_34	  (1 << 21)	/* 3.30 - 3.40 */
#define AT91C_VDD_34_35	  (1 << 22)	/* 3.40 - 3.50 */
#define AT91C_VDD_35_36	  (1 << 23)	/* 3.50 - 3.60 */
/* LOW - the card has not finished the power up routine */
#define AT91C_CARD_POWER_UP_BUSY	 (0x80000000)

#define AT91C_MMC_HOST_VOLTAGE_RANGE \
	(AT91C_VDD_27_28 +\
	AT91C_VDD_28_29  +\
	AT91C_VDD_29_30  +\
	AT91C_VDD_30_31  +\
	AT91C_VDD_31_32  +\
	AT91C_VDD_32_33)

////////////////////////////////////////////////////////////////////////////////////////////////////
// CURRENT_STATE & READY_FOR_DATA in SDCard Status Register definition (response type R1)
////////////////////////////////////////////////////////////////////////////////////////////////////
/* the command arg was out of allowed range for the card */
#define AT91C_SR_OUT_OF_RANGE       (1 << 31)
#define AT91C_SR_ADDRESS_ERROR      (1 << 30)	/* misaligned address */
#define AT91C_SR_BLOCK_LEN_ERROR    (1 << 29)	/* transferred block length error */
#define AT91C_SR_ERASE_SEQ_ERROR    (1 << 28) 	/* error in the sequence of erase */
#define AT91C_SR_ERASE_PARAM        (1 << 27)	/* an invalid selection of erase groups for erase command */
#define AT91C_SR_WP_VIOLATION       (1 << 26)	/* attempt to program a write protected block */
#define AT91C_SR_CARD_IS_LOCKED	    (1 << 25)	/* card is locked */
#define AT91C_SR_LOCK_UNLOCK_FAILED (1 << 24)	/* */
#define AT91C_SR_COM_CRC_ERROR      (1 << 23)	/* The CRC check of the previous command failed */
#define AT91C_SR_ILLEGAL_COMMAND    (1 << 22)	/* command not legal for the card state */
#define AT91C_SR_CARD_ECC_FAILED    (1 << 21)	/* card internal ECC was applied but failed to correct the data */
#define AT91C_SR_CC_ERROR           (1 << 20)	/* card controller error */
#define AT91C_SR_ERROR              (1 << 19)	/* general or an unknown error */
#define AT91C_SR_UNDERRUN           (1 << 18)	/* */
#define AT91C_SR_OVERRUN	    (1 << 17)	/* */
#define AT91C_SR_CID_CSD_OVERWRITE  (1 << 16)	
#define AT91C_SR_WP_ERASE_SKIP	    (1 << 15)	
	/* only partial address space was erased due to existing write */
	/* protected blocks */
#define AT91C_SR_CARD_ECC_DISABLED  (1 << 14)
#define AT91C_SR_ERASE_RESET	    (1 << 13)
	/* an erase sequence was cleared before executing because an out */
	/* of erase sequence command was received */
#define AT91C_SR_STATE_MASK 	   (0xF << 9) 

#define AT91C_SR_IDLE		     (0 << 9)	/* idle state */
#define AT91C_SR_READY               (1 << 9)	/* ready state */
#define AT91C_SR_IDENT               (2 << 9)	/* identification state */
#define AT91C_SR_STBY                (3 << 9)	/* standby state */
#define AT91C_SR_TRAN                (4 << 9)	/* transfer state */
#define AT91C_SR_DATA                (5 << 9)	/* sending date state */
#define AT91C_SR_RCV                 (6 << 9)	/* receiving data state */
#define AT91C_SR_PRG                 (7 << 9)	/* programming state */
#define AT91C_SR_DIS                 (8 << 9)	/* disconnect state */ 

#define AT91C_SR_READY_FOR_DATA	     (1 << 8)	// corresponds to buffer empty signalling on the bus

#define AT91C_SR_APP_CMD             (1 << 5)

/* selected mode  */
#define AT91C_SR_CARD_SELECTED	\
	(AT91C_SR_READY_FOR_DATA + AT91C_SR_TRAN)

/////////////////////////////////////////////////////////////////////////////////////////////////////
// MMC CSD register header File					
// AT91C_CSD_xxx_S	for shift value
// AT91C_CSD_xxx_M	for mask  value
/////////////////////////////////////////////////////////////////////////////////////////////////////

// First Response INT <=> CSD[3] : bits 0 to 31
#define	AT91C_CSD_BIT0_S	       0 // [0:0]			
#define	AT91C_CSD_BIT0_M	    0x01				
#define	AT91C_CSD_CRC_S		       1 // [7:1]
#define	AT91C_CSD_CRC_M		    0x7F
#define	AT91C_CSD_MMC_ECC_S	       8 // [9:8]		reserved for MMC compatibility
#define	AT91C_CSD_MMC_ECC_M	    0x03
#define	AT91C_CSD_FILE_FMT_S	      10 // [11:10]
#define	AT91C_CSD_FILE_FMT_M	    0x03
#define	AT91C_CSD_TMP_WP_S	      12 // [12:12]
#define	AT91C_CSD_TMP_WP_M	    0x01
#define	AT91C_CSD_PERM_WP_S 	      13 // [13:13]
#define	AT91C_CSD_PERM_WP_M 	    0x01
#define	AT91C_CSD_COPY_S	      14 // [14:14]
#define	AT91C_CSD_COPY_M 	    0x01
#define	AT91C_CSD_FILE_FMT_GRP_S      15 // [15:15]
#define	AT91C_CSD_FILE_FMT_GRP_M    0x01
//	reserved		      16 // [20:16]
//	reserved		    0x1F
#define	AT91C_CSD_WBLOCK_P_S 	      21 // [21:21]
#define	AT91C_CSD_WBLOCK_P_M 	    0x01
#define	AT91C_CSD_WBLEN_S 	      22 // [25:22]
#define	AT91C_CSD_WBLEN_M 	    0x0F
#define	AT91C_CSD_R2W_F_S 	      26 // [28:26]
#define	AT91C_CSD_R2W_F_M 	    0x07
#define	AT91C_CSD_MMC_DEF_ECC_S	      29 // [30:29]		reserved for MMC compatibility
#define	AT91C_CSD_MMC_DEF_ECC_M	    0x03
#define	AT91C_CSD_WP_GRP_EN_S	      31 // [31:31]
#define	AT91C_CSD_WP_GRP_EN_M 	    0x01

// Seconde Response INT <=> CSD[2] : bits 32 to 63
#define	AT91C_CSD_v21_WP_GRP_SIZE_S    0 // [38:32]				
#define	AT91C_CSD_v21_WP_GRP_SIZE_M 0x7F				
#define	AT91C_CSD_v21_SECT_SIZE_S      7 // [45:39]
#define	AT91C_CSD_v21_SECT_SIZE_M   0x7F
#define	AT91C_CSD_v21_ER_BLEN_EN_S    14 // [46:46]
#define	AT91C_CSD_v21_ER_BLEN_EN_M  0x01

#define	AT91C_CSD_v22_WP_GRP_SIZE_S    0 // [36:32]				
#define	AT91C_CSD_v22_WP_GRP_SIZE_M 0x1F				
#define	AT91C_CSD_v22_ER_GRP_SIZE_S    5 // [41:37]
#define	AT91C_CSD_v22_ER_GRP_SIZE_M 0x1F
#define	AT91C_CSD_v22_SECT_SIZE_S     10 // [46:42]
#define	AT91C_CSD_v22_SECT_SIZE_M   0x1F

#define	AT91C_CSD_C_SIZE_M_S	      15 // [49:47]
#define	AT91C_CSD_C_SIZE_M_M	    0x07
#define	AT91C_CSD_VDD_WMAX_S 	      18 // [52:50]
#define	AT91C_CSD_VDD_WMAX_M 	    0x07
#define	AT91C_CSD_VDD_WMIN_S	      21 // [55:53]
#define	AT91C_CSD_VDD_WMIN_M 	    0x07
#define	AT91C_CSD_RCUR_MAX_S 	      24 // [58:56]
#define	AT91C_CSD_RCUR_MAX_M 	    0x07
#define	AT91C_CSD_RCUR_MIN_S 	      27 // [61:59]
#define	AT91C_CSD_RCUR_MIN_M        0x07
#define	AT91C_CSD_CSIZE_L_S 	      30 // [63:62] <=> 2 LSB of CSIZE
#define	AT91C_CSD_CSIZE_L_M 	    0x03

// Third Response INT <=> CSD[1] : bits 64 to 95
#define	AT91C_CSD_CSIZE_H_S 	       0 // [73:64]	<=> 10 MSB of CSIZE
#define	AT91C_CSD_CSIZE_H_M 	  0x03FF
// reserved			      10 // [75:74]
// reserved			    0x03		
#define	AT91C_CSD_DSR_I_S 	      12 // [76:76]
#define	AT91C_CSD_DSR_I_M 	    0x01
#define	AT91C_CSD_RD_B_MIS_S 	      13 // [77:77]
#define	AT91C_CSD_RD_B_MIS_M 	    0x01
#define	AT91C_CSD_WR_B_MIS_S 	      14 // [78:78]
#define	AT91C_CSD_WR_B_MIS_M 	    0x01
#define	AT91C_CSD_RD_B_PAR_S 	      15 // [79:79]
#define	AT91C_CSD_RD_B_PAR_M 	    0x01
#define	AT91C_CSD_RD_B_LEN_S 	      16 // [83:80]
#define	AT91C_CSD_RD_B_LEN_M 	    0x0F
#define	AT91C_CSD_CCC_S	 	      20 // [95:84]
#define	AT91C_CSD_CCC_M 	  0x0FFF

// Fourth Response INT <=> CSD[0] : bits 96 to 127
#define	AT91C_CSD_TRANS_SPEED_S        0 // [103:96]
#define	AT91C_CSD_TRANS_SPEED_M     0xFF
#define	AT91C_CSD_NSAC_S 	       8 // [111:104]
#define	AT91C_CSD_NSAC_M 	    0xFF
#define	AT91C_CSD_TAAC_S 	      16 // [119:112]
#define	AT91C_CSD_TAAC_M 	    0xFF
//	reserved		      24 // [121:120]
//	reserved		    0x03
#define	AT91C_CSD_MMC_SPEC_VERS_S     26 // [125:122]	reserved for MMC compatibility
#define	AT91C_CSD_MMC_SPEC_VERS_M   0x0F
#define	AT91C_CSD_STRUCT_S            30 // [127:126]
#define	AT91C_CSD_STRUCT_M 	    0x03

/////////////////////////////////////////////////////////////////////////////////////////////////////

#endif